// This file contains all of the hardware information.
// for PCB BV503_i2_b

constant HARDWARE 1 // indicater that it is loaded and start of this file
// control port and baud rate, baud rate set by config
constant CPORT 1    // UART to use for control
constant CBAUD 115200
// this pin places the wi-fi module into mode 0 or mode 1
constant ESPIN  9   // RD9
constant ESPORT PORTD
constant LF 10
// LEDS
constant LEDPORT    PORTB
constant LEDCON     12
constant LEDERR     11
constant LEDL1      10
constant LEDL2      9
// relays
constant RLYPORT1   PORTB
constant RLYPORT2   PORTD
// LCD
// in a separate file as this may not be fitted

// *****************************************************************************
// set i/o as required
// *****************************************************************************
function init_hardware()
dim j
    com1_open(CBAUD,800) // open/reopen with bigger buffer
    // pin for changing wi fi mode
    io_write(ESPORT,ESPIN,HIGH) // es/RST active low
    io_pinMode(ESPORT,ESPIN,OUT,WOFF)
    // led outputs
    io_write(LEDPORT,LEDCON,HIGH) // leds high is off
    io_write(LEDPORT,LEDERR,HIGH)
    io_write(LEDPORT,LEDL1,HIGH)
    io_write(LEDPORT,LEDL2,HIGH)
    //
    io_pinMode(LEDPORT,LEDCON,OUT,WOFF)
    io_pinMode(LEDPORT,LEDERR,OUT,WOFF)
    io_pinMode(LEDPORT,LEDL1,OUT,WOFF)
    io_pinMode(LEDPORT,LEDL2,OUT,WOFF)
    // relay  banks 1 & 2
    for j = 0 to 7
        io_write(RLYPORT1,j,HIGH) // relays high is off
        io_write(RLYPORT2,j,HIGH)
        io_pinMode(RLYPORT1,j,OUT,WOFF)
        io_pinMode(RLYPORT2,j,OUT,WOFF)
    next 
    // LCD  ********* only if installed
    lcd_initPorts()
    lcd_init()
endf

//******************************************************************************
// translates a 0 to HIGH and 1 to LOW 
//******************************************************************************
function getHL(oon)
    if oon = 0 then
        oon = HIGH
    else
        oon = LOW
    endif
    return oon
endf    

//******************************************************************************
// led indicators 0 = off 1 = onn
//******************************************************************************
function ledCon(oon)
    io_write(LEDPORT,LEDCON,getHL(oon))
endf    
function ledErr(oon)
    io_write(LEDPORT,LEDERR,getHL(oon))
endf    
function ledL1(oon)
    io_write(LEDPORT,LEDL1,getHL(oon))
endf    
function ledL2(oon)
    io_write(LEDPORT,LEDL2,getHL(oon))
endf    

//******************************************************************************
// relay control
//******************************************************************************
function relay1(rly, oon)
    rly = max(rly,0) 
    rly = min(rly,7)
    io_write(RLYPORT1,rly,getHL(oon))
endf
function relay2(rly, oon)
    rly = max(rly,0) 
    rly = min(rly,7)
    io_write(RLYPORT2,rly,getHL(oon))
endf

