// Version for BV503 - to be loaded from SD Card, no # values
// This code is common to all MX3 devices
// Load this and save to flash. It is intended for first time users of ByPic
// that makes it much simpler for basic tasks to be carried out
// Processor MX3, Version 2.0
constant ROOKIE1 1  // indicater that it is loaded and start of this file 
constant VERSION$ "2.2"
// Version History
// 13 April 2.1  * increased I2C time out to 200000
//				* added version constant
//
//
// 2 June 2.2
//  * Added com1_open and com1_pins to make com1 device independant
//
constant CLR    4
constant SET    8
constant INV    12
//
// =============================================================================
// General purpose I/O ports
//
constant OUT    CLR
constant IN     SET
constant HIGH   SET
constant LOW    CLR
constant TOGGLE INV
constant WOFF   0 // waek pull up / down off
constant WPU    1 // weak pull up
// ADC
constant AD1PCFG          0xBF809060
//
constant PORTB          0xBF886040 // TRISB
constant PORTC          0xBF886080 // TRISC
constant PORTD          0xBF8860C0 // TRISD
constant PORTE          0xBF886100 // TRISE
constant PORTF          0xBF886140 // TRISF
constant PORTG          0xBF886180 // TRISG
// change notics and pull ups are limited on the MX3
constant CNCON            0xBF8861C0
constant CNEN             0xBF8861D0
constant CNPUE            0xBF8861E0
// have to map change notice and pull up to requested port
//  offsets from TRIS
constant TRIS_OFF   0
constant PORT_OFF   0x10
constant LAT_OFF    0x20

// *****************************************************************************
// pull up and change notice are intertwined and more complex than MX1, 64 pin
// devices do not have RD13 to RD15. Will also set the change notice here. It
// will not have an effect until the CNCON bit is enabled by interrupts
// action is either SET ot CLR
// *****************************************************************************
function io__weakPullUp(port, pin, action)
    select(port)
        case(PORTB)
            if pin = 15 then
                poke(CNPUE+action, 1 << 12)
                poke(CNEN+action, 1 << 12)
            endif
            if pin < 10 then
                poke(CNPUE+action, 1 << (pin+2))
                poke(CNEN+action, 1 << (pin+2))
            endif
        case(PORTC)
            if pin = 13 then
                poke(CNPUE+action, 1 << 1)
                poke(CNEN+action, 1 << 1)
            endif
            if pin = 14 then
                poke(CNPUE+action, 1 )
                poke(CNEN+action, 1 )
            endif
        case(PORTD)
            if (pin > 3) && (pin < 8) then
                poke(CNPUE+action, 1 << (pin+9))
                poke(CNEN+action, 1 << (pin+9))
            endif
        case(PORTF)
            if (pin = 4) || (pin = 5) then
                poke(CNPUE+action, 1 << (pin+13))
                poke(CNEN+action, 1 << (pin+13))
            endif
    endselect
endf

// *****************************************************************************
// sets i/o pin to either input or output
// Example: set RB5 to be an output
// io_pinMode(PORTB,5,OUT, weak_pull_up)
// *****************************************************************************
function io_pinMode(port,pin,ionorout, pull)
    if port >= PORTB then // simple check
        // adc only on port b
        if port = PORTB then
            poke(AD1PCFG+SET,1 << pin) // also need to set ADC off
        endif
        poke(port+TRIS_OFF+ionorout,1 << pin) // set TRISn
        // wealk pouu ups
        if pull = WPU
            io__weakPullUp(port, pin, SET)
        else
            io__weakPullUp(port, pin, CLR)
        endif
    else
        print "\nERROR - wrong port value"
    endif
endf

// *****************************************************************************
// reads a pin on a port
// Example: read pin RB14
// x = io_read(PORTB,14)
// *****************************************************************************
function io_read(port,pin)
dim k
    k = peek(port+PORT_OFF) // PORTn
    return (k >> pin) & 0x1
endf

// *****************************************************************************
// sets a particular pin on a port
// Example: set pin RB14 to 1
// io_write(PORTB,14,HIGH)
// io_write(PRTB,14,TOGGLE) // now makes it ZERO
// *****************************************************************************
function io_write(port,pin,value)
    poke(port+LAT_OFF+value,1 << pin)
endf

// *****************************************************************************
// gets a whole port into a 32 bit integer
// Example:
// x = io_getPort(PORTA)
// *****************************************************************************
function io_getPort(port)
    return peek(port+PORT_OFF)
endf

// *****************************************************************************
// sets a whole port to a value
// Example set bits 0 and 1 high and the rest low of port B
// io_setPort(PORTB,3)
// *****************************************************************************
function io_setPort(port,value)
    poke(port+LAT_OFF,value)
endf

// *****************************************************************************
// sets or clears only the bits that are set on the value
// *****************************************************************************
function io_setPortH(port,value)
    poke(port+LAT_OFF+SET,value)
endf

function io_setPortL(port,value)
    poke(port+LAT_OFF+CLR,value)
endf

// =============================================================================
// TABLES
// The following allow simple manipulation of tables
// *****************************************************************************
// The table must be supplied as an address for example if a table constant
// is TBL$ {1,2,3,45}
// then to get the value of 2 use v = tbl_get$(?TBL$,1)
// Obviously the replace functions will not work with values in flash
// *****************************************************************************
function tbl_size(table)
    return peek(table)
endf
function tbl_get$(table, offset)
    return peekc(table+4+offset)
endf
function tbl_get(table, offset)
    return peek(table+(offset*4))
endf
function tbl_replace$(table, offset, value)
    pokec(table+4+offset,value)
endf
function tbl_replace(table, offset, value)
    poke(table+(offset*4),value)
endf

// =============================================================================
// ADC section, MX3, port B is dedicated to ADC
//
constant SAMPLE 5
constant CONVERSION 5
//
constant AD1CON1         0xBF809000
constant AD1CON2         0xBF809010
constant AD1CON3         0xBF809020
constant AD1CHS          0xBF809040
constant AD1CSSL         0xBF809050
constant ADC1BUF0        0xBF809070
//
constant TRISB            0xBF886040
// *****************************************************************************
// Set up ADC for auto ample and scan
// channel number is 0 to 9, set up one channel at once
// *****************************************************************************
function adc_init(chan)
dim map, ch
    poke(AD1PCFG+CLR,1 << chan) // set to ADC
    poke(TRISB+SET,1 << chan) // make sure its an input
    @AD1CON3 = 0 // reset first, clock derived from PBCLK
    @AD1CON3 = (SAMPLE << 8) | CONVERSION
    @AD1CON2 = 0
    @AD1CON1 = 0x80e0 // auto conversion + turn on ADC
    @AD1CSSL = 0
endf
// *****************************************************************************
// sample selected channel and return value
// *****************************************************************************
function adc_get(channel)
    @AD1CHS = channel << 16
    poke(AD1CON1+SET, 2) // start conversion
    while (peek(AD1CON1) & 1) = 0; wend
    return @ADC1BUF0
endf

// =============================================================================
// Timers section

constant T3IF 1 << 12
// Timer regiasters
constant T2CON           0xBF800800
constant TMR2            0xBF800810
constant PR2             0xBF800820
constant T4CON            0xBF800C00
constant TMR4             0xBF800C10
constant PR4              0xBF800C20
// Interrupt registers
constant IFS0            0xBF881030
constant TIMER23 {12, 1 << 12, TMR2, PR2, T2CON, IFS0}
constant TIMER45 {20, 1 << 20, TMR4, PR4, T4CON, IFS0}
// *****************************************************************************
// uses t2 & 3 to form a 32 bit timer, simple to use for general purpose
// work. This sets the time base
// *****************************************************************************
function tmr_set(timer, value)
dim v
	v = value * 10 // prescale 4 is 0.1uS
    poke(tbl_get(timer,2), 0)  // clear counter
    poke(tbl_get(timer,5)+CLR,tbl_get(timer,1)) // clear flag
    // timer_on | prescale_4 | 32_bit_on | Internal_clock
    poke(tbl_get(timer,4), 0x8028)
    poke(tbl_get(timer,3), v)
endf

function tmr_get(timer)
	return peek(tbl_get(timer,2))
endf

function tmr_flag(timer)
	return  (peek(tbl_get(timer,5)) & tbl_get(timer,1)) >> tbl_get(timer,0)
endf

function tmr_clr(timer)
	poke(tbl_get(timer,2), 0)
	poke(tbl_get(timer,5)+CLR,tbl_get(timer,1)) // clear flag
endf

function tmr_s(timer, start)
    if start <> 0 then
        poke(tbl_get(timer,4)+SET,0x8000)
    else
        poke(tbl_get(timer,4)+CLR,0x8000)
    endif
endf

// =============================================================================
// I2C Section. MX3 devices have this built in so this is just a matter of
// translating the built in functions to the standard ones
// Hardware
// SCL1 is RG2
// SDA1 is RG3
//
constant I2CTIMEOUT 200000
// ============================= I2C Channel 1 =================================
// The functions here are set up for channel 1.
//
// *****************************************************************************
// Open i2c, specify rate in Hz
// *****************************************************************************
function i2c_open(rate)
    i2copen(1,rate)
endf

// *****************************************************************************
// Close
// *****************************************************************************
function i2c_close()
    i2cclose(1)
endf

// *****************************************************************************
// send re-start condition, slightly different format to mx1
// *****************************************************************************
function i2c_rstart(adr)
    i2crestart(1,adr)
endf

// *****************************************************************************
// send stop condition
// *****************************************************************************
function i2c_stop()
    i2cstop(1)
endf

// *****************************************************************************
// searches bus for I2C device, returns when address is found, returns with the
// found address.
// specify start and end address
// *****************************************************************************
function i2c_find(start,end)
    return i2cscan(1,start,end)
endf

// *****************************************************************************
// send byte to i2c
// returns     -1 on error
//            1 if NACK recieved (normal)
//            0 if ACK received
// *****************************************************************************
function i2c_putc(c)
    i2cputc(1,c)
endf

// *****************************************************************************
// send start condition and address
// *****************************************************************************
function i2c_start(adr)
    i2cstart(1,adr)
endf

// *****************************************************************************
// recieve byte from I2C, error indiocation by print
// *****************************************************************************
function i2c_getc(last)
dim rv,v
    rv = i2cgetc(1,?v,I2CTIMEOUT,last)
    if rv <> 0 then
       	print "Error on I2C getc"
        return 0
    else
        return v
    endif
endf

// =============================================================================
// SPI Section, this is built into the MX3 to drive the SD Card, utilizes
// channel 2 as channel 1 is also UART1 that is probably more useful
//
// Hardware for SPI
// Data out SDO2 is RG8 (MOSI)
// Data in  SDI2 is RG7 (MISO)
// Clock SCK1 is RG6
//
// *****************************************************************************
// Already anitialised on start up at 10MHz so this is used to set a different
// speed
// MAX is 20MHz
// *****************************************************************************
function spi_init(speed)
    spirate(speed)
endf

// *****************************************************************************
// main data transfer call to read or write, when reading set v to -1
// *****************************************************************************
function spi_data(v)
    return spi(v)
endf

// =============================================================================
// INTERRUPTS
//
constant IFS0            0xBF881030
constant IFS1            0xBF881040
constant IPC0            0xBF881090
constant IPC3            0xBF8810C0
constant IPC5            0xBF8810E0
constant IPC6            0xBF8810F0
constant IPC8            0xBF881110
constant IEC0            0xBF881060
constant IEC1            0xBF881070
// Interrupt tables, some typical ones
// interrupt table contains 3 values:
// id, flag_mask, priority_mask, flag_register, enable_register, interrupt_register
// The ID is for the IrFlag and ID below 9 is a port
// First 8 locations are for pin change notice
// These are other types of interrupt
constant IrTIMER23 {9, 1 << 12, 3 << 2, IFS0, IEC0, IPC3 }
constant IrTIMER45 {10, 1 << 20, 3 << 2, IFS0, IEC0, IPC5 } // timer45
constant IrEXT0 {11, 1 << 3, 3 << 26, IFS0, IEC0, IPC0 } // external int 0
dim IrFlag // so that interrups cant be set twice

// *****************************************************************************
// Clears the interrupt table and also the flag used in rookie so that ir_set
// can be called more than once without causing problems
// *****************************************************************************
function ir_clear()
    IrFlag = 0
    irclear()
endf

// *****************************************************************************
// given an interrupt table, that has all of the IR information, this will
// associate an interrupt with a function. This does indeed simplify the
// process of setting interrupts.
// *****************************************************************************
function ir_set(interrupt, func$[40])
dim iflag
    iflag = 1 << tbl_get(interrupt, 0)
    if (iflag & IrFlag) = 0 then
        IrFlag = IrFlag | iflag // set flag for next time
        poke(tbl_get(interrupt,3)+CLR,tbl_get(interrupt,1)) // clear interrupt
        poke(tbl_get(interrupt,5)+SET,tbl_get(interrupt,2)) // set priority
        poke(tbl_get(interrupt,4)+SET,tbl_get(interrupt,1)) // enable
        irset(tbl_get(interrupt,3),tbl_get(interrupt,1),func$) // table entry
    endif
endf

// *****************************************************************************
// given an interrupt table, that has all of the IR information, this will
// associate an interrupt with a function. This one is for a pin change
// which is slightly different
// The MX3 just has certain pins that can be used as interrupt on change
// *****************************************************************************
function ir_setPin(port, func$[40], pin)
    @CNCON = 1 << 15 // enable
    io_getPort(port)
    io_pinMode(port, pin, IN, WPU)  // this will set input and also CNEN
    poke(IPC6+SET, 3 << 18) // priority
    irset(IFS1, 1, func$)
    // essential otherwise you gen an interrupt at start
    io_getPort(port)
    poke(IFS1+CLR,1) // clear
    poke(IEC1+SET,1) // enable
endf

// =============================================================================
// COM Port pins
// The MX1 has the option of specifying which pins can be used but the MX3
// does not but to make com1 cross platform, use this Function
// *****************************************************************************
// opens com port 1 with default pins
// *****************************************************************************
function com1_open(baud,buffer)
    comopen(1,baud,buffer)
endf


